# Generated by Snowball 2.2.0 - https://snowballstem.org/

from .basestemmer import BaseStemmer
from .among import Among


class SpanishStemmer(BaseStemmer):
    '''
    This class implements the stemming algorithm defined by a snowball script.
    Generated by Snowball 2.2.0 - https://snowballstem.org/
    '''

    a_0 = [
        Among(u"", -1, 6),
        Among(u"\u00E1", 0, 1),
        Among(u"\u00E9", 0, 2),
        Among(u"\u00ED", 0, 3),
        Among(u"\u00F3", 0, 4),
        Among(u"\u00FA", 0, 5)
    ]

    a_1 = [
        Among(u"la", -1, -1),
        Among(u"sela", 0, -1),
        Among(u"le", -1, -1),
        Among(u"me", -1, -1),
        Among(u"se", -1, -1),
        Among(u"lo", -1, -1),
        Among(u"selo", 5, -1),
        Among(u"las", -1, -1),
        Among(u"selas", 7, -1),
        Among(u"les", -1, -1),
        Among(u"los", -1, -1),
        Among(u"selos", 10, -1),
        Among(u"nos", -1, -1)
    ]

    a_2 = [
        Among(u"ando", -1, 6),
        Among(u"iendo", -1, 6),
        Among(u"yendo", -1, 7),
        Among(u"\u00E1ndo", -1, 2),
        Among(u"i\u00E9ndo", -1, 1),
        Among(u"ar", -1, 6),
        Among(u"er", -1, 6),
        Among(u"ir", -1, 6),
        Among(u"\u00E1r", -1, 3),
        Among(u"\u00E9r", -1, 4),
        Among(u"\u00EDr", -1, 5)
    ]

    a_3 = [
        Among(u"ic", -1, -1),
        Among(u"ad", -1, -1),
        Among(u"os", -1, -1),
        Among(u"iv", -1, 1)
    ]

    a_4 = [
        Among(u"able", -1, 1),
        Among(u"ible", -1, 1),
        Among(u"ante", -1, 1)
    ]

    a_5 = [
        Among(u"ic", -1, 1),
        Among(u"abil", -1, 1),
        Among(u"iv", -1, 1)
    ]

    a_6 = [
        Among(u"ica", -1, 1),
        Among(u"ancia", -1, 2),
        Among(u"encia", -1, 5),
        Among(u"adora", -1, 2),
        Among(u"osa", -1, 1),
        Among(u"ista", -1, 1),
        Among(u"iva", -1, 9),
        Among(u"anza", -1, 1),
        Among(u"log\u00EDa", -1, 3),
        Among(u"idad", -1, 8),
        Among(u"able", -1, 1),
        Among(u"ible", -1, 1),
        Among(u"ante", -1, 2),
        Among(u"mente", -1, 7),
        Among(u"amente", 13, 6),
        Among(u"aci\u00F3n", -1, 2),
        Among(u"uci\u00F3n", -1, 4),
        Among(u"ico", -1, 1),
        Among(u"ismo", -1, 1),
        Among(u"oso", -1, 1),
        Among(u"amiento", -1, 1),
        Among(u"imiento", -1, 1),
        Among(u"ivo", -1, 9),
        Among(u"ador", -1, 2),
        Among(u"icas", -1, 1),
        Among(u"ancias", -1, 2),
        Among(u"encias", -1, 5),
        Among(u"adoras", -1, 2),
        Among(u"osas", -1, 1),
        Among(u"istas", -1, 1),
        Among(u"ivas", -1, 9),
        Among(u"anzas", -1, 1),
        Among(u"log\u00EDas", -1, 3),
        Among(u"idades", -1, 8),
        Among(u"ables", -1, 1),
        Among(u"ibles", -1, 1),
        Among(u"aciones", -1, 2),
        Among(u"uciones", -1, 4),
        Among(u"adores", -1, 2),
        Among(u"antes", -1, 2),
        Among(u"icos", -1, 1),
        Among(u"ismos", -1, 1),
        Among(u"osos", -1, 1),
        Among(u"amientos", -1, 1),
        Among(u"imientos", -1, 1),
        Among(u"ivos", -1, 9)
    ]

    a_7 = [
        Among(u"ya", -1, 1),
        Among(u"ye", -1, 1),
        Among(u"yan", -1, 1),
        Among(u"yen", -1, 1),
        Among(u"yeron", -1, 1),
        Among(u"yendo", -1, 1),
        Among(u"yo", -1, 1),
        Among(u"yas", -1, 1),
        Among(u"yes", -1, 1),
        Among(u"yais", -1, 1),
        Among(u"yamos", -1, 1),
        Among(u"y\u00F3", -1, 1)
    ]

    a_8 = [
        Among(u"aba", -1, 2),
        Among(u"ada", -1, 2),
        Among(u"ida", -1, 2),
        Among(u"ara", -1, 2),
        Among(u"iera", -1, 2),
        Among(u"\u00EDa", -1, 2),
        Among(u"ar\u00EDa", 5, 2),
        Among(u"er\u00EDa", 5, 2),
        Among(u"ir\u00EDa", 5, 2),
        Among(u"ad", -1, 2),
        Among(u"ed", -1, 2),
        Among(u"id", -1, 2),
        Among(u"ase", -1, 2),
        Among(u"iese", -1, 2),
        Among(u"aste", -1, 2),
        Among(u"iste", -1, 2),
        Among(u"an", -1, 2),
        Among(u"aban", 16, 2),
        Among(u"aran", 16, 2),
        Among(u"ieran", 16, 2),
        Among(u"\u00EDan", 16, 2),
        Among(u"ar\u00EDan", 20, 2),
        Among(u"er\u00EDan", 20, 2),
        Among(u"ir\u00EDan", 20, 2),
        Among(u"en", -1, 1),
        Among(u"asen", 24, 2),
        Among(u"iesen", 24, 2),
        Among(u"aron", -1, 2),
        Among(u"ieron", -1, 2),
        Among(u"ar\u00E1n", -1, 2),
        Among(u"er\u00E1n", -1, 2),
        Among(u"ir\u00E1n", -1, 2),
        Among(u"ado", -1, 2),
        Among(u"ido", -1, 2),
        Among(u"ando", -1, 2),
        Among(u"iendo", -1, 2),
        Among(u"ar", -1, 2),
        Among(u"er", -1, 2),
        Among(u"ir", -1, 2),
        Among(u"as", -1, 2),
        Among(u"abas", 39, 2),
        Among(u"adas", 39, 2),
        Among(u"idas", 39, 2),
        Among(u"aras", 39, 2),
        Among(u"ieras", 39, 2),
        Among(u"\u00EDas", 39, 2),
        Among(u"ar\u00EDas", 45, 2),
        Among(u"er\u00EDas", 45, 2),
        Among(u"ir\u00EDas", 45, 2),
        Among(u"es", -1, 1),
        Among(u"ases", 49, 2),
        Among(u"ieses", 49, 2),
        Among(u"abais", -1, 2),
        Among(u"arais", -1, 2),
        Among(u"ierais", -1, 2),
        Among(u"\u00EDais", -1, 2),
        Among(u"ar\u00EDais", 55, 2),
        Among(u"er\u00EDais", 55, 2),
        Among(u"ir\u00EDais", 55, 2),
        Among(u"aseis", -1, 2),
        Among(u"ieseis", -1, 2),
        Among(u"asteis", -1, 2),
        Among(u"isteis", -1, 2),
        Among(u"\u00E1is", -1, 2),
        Among(u"\u00E9is", -1, 1),
        Among(u"ar\u00E9is", 64, 2),
        Among(u"er\u00E9is", 64, 2),
        Among(u"ir\u00E9is", 64, 2),
        Among(u"ados", -1, 2),
        Among(u"idos", -1, 2),
        Among(u"amos", -1, 2),
        Among(u"\u00E1bamos", 70, 2),
        Among(u"\u00E1ramos", 70, 2),
        Among(u"i\u00E9ramos", 70, 2),
        Among(u"\u00EDamos", 70, 2),
        Among(u"ar\u00EDamos", 74, 2),
        Among(u"er\u00EDamos", 74, 2),
        Among(u"ir\u00EDamos", 74, 2),
        Among(u"emos", -1, 1),
        Among(u"aremos", 78, 2),
        Among(u"eremos", 78, 2),
        Among(u"iremos", 78, 2),
        Among(u"\u00E1semos", 78, 2),
        Among(u"i\u00E9semos", 78, 2),
        Among(u"imos", -1, 2),
        Among(u"ar\u00E1s", -1, 2),
        Among(u"er\u00E1s", -1, 2),
        Among(u"ir\u00E1s", -1, 2),
        Among(u"\u00EDs", -1, 2),
        Among(u"ar\u00E1", -1, 2),
        Among(u"er\u00E1", -1, 2),
        Among(u"ir\u00E1", -1, 2),
        Among(u"ar\u00E9", -1, 2),
        Among(u"er\u00E9", -1, 2),
        Among(u"ir\u00E9", -1, 2),
        Among(u"i\u00F3", -1, 2)
    ]

    a_9 = [
        Among(u"a", -1, 1),
        Among(u"e", -1, 2),
        Among(u"o", -1, 1),
        Among(u"os", -1, 1),
        Among(u"\u00E1", -1, 1),
        Among(u"\u00E9", -1, 2),
        Among(u"\u00ED", -1, 1),
        Among(u"\u00F3", -1, 1)
    ]

    g_v = [17, 65, 16, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 17, 4, 10]

    I_p2 = 0
    I_p1 = 0
    I_pV = 0

    def __r_mark_regions(self):
        self.I_pV = self.limit
        self.I_p1 = self.limit
        self.I_p2 = self.limit
        v_1 = self.cursor
        try:
            try:
                v_2 = self.cursor
                try:
                    if not self.in_grouping(SpanishStemmer.g_v, 97, 252):
                        raise lab2()
                    try:
                        v_3 = self.cursor
                        try:
                            if not self.out_grouping(SpanishStemmer.g_v, 97, 252):
                                raise lab4()
                            if not self.go_out_grouping(SpanishStemmer.g_v, 97, 252):
                                raise lab4()
                            self.cursor += 1
                            raise lab3()
                        except lab4: pass
                        self.cursor = v_3
                        if not self.in_grouping(SpanishStemmer.g_v, 97, 252):
                            raise lab2()
                        if not self.go_in_grouping(SpanishStemmer.g_v, 97, 252):
                            raise lab2()
                        self.cursor += 1
                    except lab3: pass
                    raise lab1()
                except lab2: pass
                self.cursor = v_2
                if not self.out_grouping(SpanishStemmer.g_v, 97, 252):
                    raise lab0()
                try:
                    v_4 = self.cursor
                    try:
                        if not self.out_grouping(SpanishStemmer.g_v, 97, 252):
                            raise lab6()
                        if not self.go_out_grouping(SpanishStemmer.g_v, 97, 252):
                            raise lab6()
                        self.cursor += 1
                        raise lab5()
                    except lab6: pass
                    self.cursor = v_4
                    if not self.in_grouping(SpanishStemmer.g_v, 97, 252):
                        raise lab0()
                    if self.cursor >= self.limit:
                        raise lab0()
                    self.cursor += 1
                except lab5: pass
            except lab1: pass
            self.I_pV = self.cursor
        except lab0: pass
        self.cursor = v_1
        v_5 = self.cursor
        try:
            if not self.go_out_grouping(SpanishStemmer.g_v, 97, 252):
                raise lab7()
            self.cursor += 1
            if not self.go_in_grouping(SpanishStemmer.g_v, 97, 252):
                raise lab7()
            self.cursor += 1
            self.I_p1 = self.cursor
            if not self.go_out_grouping(SpanishStemmer.g_v, 97, 252):
                raise lab7()
            self.cursor += 1
            if not self.go_in_grouping(SpanishStemmer.g_v, 97, 252):
                raise lab7()
            self.cursor += 1
            self.I_p2 = self.cursor
        except lab7: pass
        self.cursor = v_5
        return True

    def __r_postlude(self):
        while True:
            v_1 = self.cursor
            try:
                self.bra = self.cursor
                among_var = self.find_among(SpanishStemmer.a_0)
                if among_var == 0:
                    raise lab0()
                self.ket = self.cursor
                if among_var == 1:
                    if not self.slice_from(u"a"):
                        return False
                elif among_var == 2:
                    if not self.slice_from(u"e"):
                        return False
                elif among_var == 3:
                    if not self.slice_from(u"i"):
                        return False
                elif among_var == 4:
                    if not self.slice_from(u"o"):
                        return False
                elif among_var == 5:
                    if not self.slice_from(u"u"):
                        return False
                else:
                    if self.cursor >= self.limit:
                        raise lab0()
                    self.cursor += 1
                continue
            except lab0: pass
            self.cursor = v_1
            break
        return True

    def __r_RV(self):
        if not self.I_pV <= self.cursor:
            return False
        return True

    def __r_R1(self):
        if not self.I_p1 <= self.cursor:
            return False
        return True

    def __r_R2(self):
        if not self.I_p2 <= self.cursor:
            return False
        return True

    def __r_attached_pronoun(self):
        self.ket = self.cursor
        if self.find_among_b(SpanishStemmer.a_1) == 0:
            return False
        self.bra = self.cursor
        among_var = self.find_among_b(SpanishStemmer.a_2)
        if among_var == 0:
            return False
        if not self.__r_RV():
            return False
        if among_var == 1:
            self.bra = self.cursor
            if not self.slice_from(u"iendo"):
                return False
        elif among_var == 2:
            self.bra = self.cursor
            if not self.slice_from(u"ando"):
                return False
        elif among_var == 3:
            self.bra = self.cursor
            if not self.slice_from(u"ar"):
                return False
        elif among_var == 4:
            self.bra = self.cursor
            if not self.slice_from(u"er"):
                return False
        elif among_var == 5:
            self.bra = self.cursor
            if not self.slice_from(u"ir"):
                return False
        elif among_var == 6:
            if not self.slice_del():
                return False

        else:
            if not self.eq_s_b(u"u"):
                return False
            if not self.slice_del():
                return False

        return True

    def __r_standard_suffix(self):
        self.ket = self.cursor
        among_var = self.find_among_b(SpanishStemmer.a_6)
        if among_var == 0:
            return False
        self.bra = self.cursor
        if among_var == 1:
            if not self.__r_R2():
                return False
            if not self.slice_del():
                return False

        elif among_var == 2:
            if not self.__r_R2():
                return False
            if not self.slice_del():
                return False

            v_1 = self.limit - self.cursor
            try:
                self.ket = self.cursor
                if not self.eq_s_b(u"ic"):
                    self.cursor = self.limit - v_1
                    raise lab0()
                self.bra = self.cursor
                if not self.__r_R2():
                    self.cursor = self.limit - v_1
                    raise lab0()
                if not self.slice_del():
                    return False

            except lab0: pass
        elif among_var == 3:
            if not self.__r_R2():
                return False
            if not self.slice_from(u"log"):
                return False
        elif among_var == 4:
            if not self.__r_R2():
                return False
            if not self.slice_from(u"u"):
                return False
        elif among_var == 5:
            if not self.__r_R2():
                return False
            if not self.slice_from(u"ente"):
                return False
        elif among_var == 6:
            if not self.__r_R1():
                return False
            if not self.slice_del():
                return False

            v_2 = self.limit - self.cursor
            try:
                self.ket = self.cursor
                among_var = self.find_among_b(SpanishStemmer.a_3)
                if among_var == 0:
                    self.cursor = self.limit - v_2
                    raise lab1()
                self.bra = self.cursor
                if not self.__r_R2():
                    self.cursor = self.limit - v_2
                    raise lab1()
                if not self.slice_del():
                    return False

                if among_var == 1:
                    self.ket = self.cursor
                    if not self.eq_s_b(u"at"):
                        self.cursor = self.limit - v_2
                        raise lab1()
                    self.bra = self.cursor
                    if not self.__r_R2():
                        self.cursor = self.limit - v_2
                        raise lab1()
                    if not self.slice_del():
                        return False

            except lab1: pass
        elif among_var == 7:
            if not self.__r_R2():
                return False
            if not self.slice_del():
                return False

            v_3 = self.limit - self.cursor
            try:
                self.ket = self.cursor
                if self.find_among_b(SpanishStemmer.a_4) == 0:
                    self.cursor = self.limit - v_3
                    raise lab2()
                self.bra = self.cursor
                if not self.__r_R2():
                    self.cursor = self.limit - v_3
                    raise lab2()
                if not self.slice_del():
                    return False

            except lab2: pass
        elif among_var == 8:
            if not self.__r_R2():
                return False
            if not self.slice_del():
                return False

            v_4 = self.limit - self.cursor
            try:
                self.ket = self.cursor
                if self.find_among_b(SpanishStemmer.a_5) == 0:
                    self.cursor = self.limit - v_4
                    raise lab3()
                self.bra = self.cursor
                if not self.__r_R2():
                    self.cursor = self.limit - v_4
                    raise lab3()
                if not self.slice_del():
                    return False

            except lab3: pass
        else:
            if not self.__r_R2():
                return False
            if not self.slice_del():
                return False

            v_5 = self.limit - self.cursor
            try:
                self.ket = self.cursor
                if not self.eq_s_b(u"at"):
                    self.cursor = self.limit - v_5
                    raise lab4()
                self.bra = self.cursor
                if not self.__r_R2():
                    self.cursor = self.limit - v_5
                    raise lab4()
                if not self.slice_del():
                    return False

            except lab4: pass
        return True

    def __r_y_verb_suffix(self):
        if self.cursor < self.I_pV:
            return False
        v_2 = self.limit_backward
        self.limit_backward = self.I_pV
        self.ket = self.cursor
        if self.find_among_b(SpanishStemmer.a_7) == 0:
            self.limit_backward = v_2
            return False
        self.bra = self.cursor
        self.limit_backward = v_2
        if not self.eq_s_b(u"u"):
            return False
        if not self.slice_del():
            return False

        return True

    def __r_verb_suffix(self):
        if self.cursor < self.I_pV:
            return False
        v_2 = self.limit_backward
        self.limit_backward = self.I_pV
        self.ket = self.cursor
        among_var = self.find_among_b(SpanishStemmer.a_8)
        if among_var == 0:
            self.limit_backward = v_2
            return False
        self.bra = self.cursor
        self.limit_backward = v_2
        if among_var == 1:
            v_3 = self.limit - self.cursor
            try:
                if not self.eq_s_b(u"u"):
                    self.cursor = self.limit - v_3
                    raise lab0()
                v_4 = self.limit - self.cursor
                if not self.eq_s_b(u"g"):
                    self.cursor = self.limit - v_3
                    raise lab0()
                self.cursor = self.limit - v_4
            except lab0: pass
            self.bra = self.cursor
            if not self.slice_del():
                return False

        else:
            if not self.slice_del():
                return False

        return True

    def __r_residual_suffix(self):
        self.ket = self.cursor
        among_var = self.find_among_b(SpanishStemmer.a_9)
        if among_var == 0:
            return False
        self.bra = self.cursor
        if among_var == 1:
            if not self.__r_RV():
                return False
            if not self.slice_del():
                return False

        else:
            if not self.__r_RV():
                return False
            if not self.slice_del():
                return False

            v_1 = self.limit - self.cursor
            try:
                self.ket = self.cursor
                if not self.eq_s_b(u"u"):
                    self.cursor = self.limit - v_1
                    raise lab0()
                self.bra = self.cursor
                v_2 = self.limit - self.cursor
                if not self.eq_s_b(u"g"):
                    self.cursor = self.limit - v_1
                    raise lab0()
                self.cursor = self.limit - v_2
                if not self.__r_RV():
                    self.cursor = self.limit - v_1
                    raise lab0()
                if not self.slice_del():
                    return False

            except lab0: pass
        return True

    def _stem(self):
        self.__r_mark_regions()
        self.limit_backward = self.cursor
        self.cursor = self.limit
        v_2 = self.limit - self.cursor
        self.__r_attached_pronoun()
        self.cursor = self.limit - v_2
        v_3 = self.limit - self.cursor
        try:
            try:
                v_4 = self.limit - self.cursor
                try:
                    if not self.__r_standard_suffix():
                        raise lab2()
                    raise lab1()
                except lab2: pass
                self.cursor = self.limit - v_4
                try:
                    if not self.__r_y_verb_suffix():
                        raise lab3()
                    raise lab1()
                except lab3: pass
                self.cursor = self.limit - v_4
                if not self.__r_verb_suffix():
                    raise lab0()
            except lab1: pass
        except lab0: pass
        self.cursor = self.limit - v_3
        v_5 = self.limit - self.cursor
        self.__r_residual_suffix()
        self.cursor = self.limit - v_5
        self.cursor = self.limit_backward
        v_6 = self.cursor
        self.__r_postlude()
        self.cursor = v_6
        return True


class lab0(BaseException): pass


class lab1(BaseException): pass


class lab2(BaseException): pass


class lab3(BaseException): pass


class lab4(BaseException): pass


class lab5(BaseException): pass


class lab6(BaseException): pass


class lab7(BaseException): pass



__all__ = ('language', 'stemmer')

from .arabic_stemmer import ArabicStemmer
from .armenian_stemmer import ArmenianStemmer
from .basque_stemmer import BasqueStemmer
from .catalan_stemmer import CatalanStemmer
from .danish_stemmer import DanishStemmer
from .dutch_stemmer import DutchStemmer
from .english_stemmer import EnglishStemmer
from .finnish_stemmer import FinnishStemmer
from .french_stemmer import FrenchStemmer
from .german_stemmer import GermanStemmer
from .greek_stemmer import GreekStemmer
from .hindi_stemmer import HindiStemmer
from .hungarian_stemmer import HungarianStemmer
from .indonesian_stemmer import IndonesianStemmer
from .irish_stemmer import IrishStemmer
from .italian_stemmer import ItalianStemmer
from .lithuanian_stemmer import LithuanianStemmer
from .nepali_stemmer import NepaliStemmer
from .norwegian_stemmer import NorwegianStemmer
from .porter_stemmer import PorterStemmer
from .portuguese_stemmer import PortugueseStemmer
from .romanian_stemmer import RomanianStemmer
from .russian_stemmer import RussianStemmer
from .serbian_stemmer import SerbianStemmer
from .spanish_stemmer import SpanishStemmer
from .swedish_stemmer import SwedishStemmer
from .tamil_stemmer import TamilStemmer
from .turkish_stemmer import TurkishStemmer
from .yiddish_stemmer import YiddishStemmer

_languages = {
    'arabic': ArabicStemmer,
    'armenian': ArmenianStemmer,
    'basque': BasqueStemmer,
    'catalan': CatalanStemmer,
    'danish': DanishStemmer,
    'dutch': DutchStemmer,
    'english': EnglishStemmer,
    'finnish': FinnishStemmer,
    'french': FrenchStemmer,
    'german': GermanStemmer,
    'greek': GreekStemmer,
    'hindi': HindiStemmer,
    'hungarian': HungarianStemmer,
    'indonesian': IndonesianStemmer,
    'irish': IrishStemmer,
    'italian': ItalianStemmer,
    'lithuanian': LithuanianStemmer,
    'nepali': NepaliStemmer,
    'norwegian': NorwegianStemmer,
    'porter': PorterStemmer,
    'portuguese': PortugueseStemmer,
    'romanian': RomanianStemmer,
    'russian': RussianStemmer,
    'serbian': SerbianStemmer,
    'spanish': SpanishStemmer,
    'swedish': SwedishStemmer,
    'tamil': TamilStemmer,
    'turkish': TurkishStemmer,
    'yiddish': YiddishStemmer,
}

try:
    import Stemmer
    cext_available = True
except ImportError:
    cext_available = False

def algorithms():
    if cext_available:
        return Stemmer.language()
    else:
        return list(_languages.keys())

def stemmer(lang):
    if cext_available:
        return Stemmer.Stemmer(lang)
    if lang.lower() in _languages:
        return _languages[lang.lower()]()
    else:
        raise KeyError("Stemming algorithm '%s' not found" % lang)



# Generated by Snowball 2.2.0 - https://snowballstem.org/

from .basestemmer import BaseStemmer
from .among import Among


class DutchStemmer(BaseStemmer):
    '''
    This class implements the stemming algorithm defined by a snowball script.
    Generated by Snowball 2.2.0 - https://snowballstem.org/
    '''

    a_0 = [
        Among(u"", -1, 6),
        Among(u"\u00E1", 0, 1),
        Among(u"\u00E4", 0, 1),
        Among(u"\u00E9", 0, 2),
        Among(u"\u00EB", 0, 2),
        Among(u"\u00ED", 0, 3),
        Among(u"\u00EF", 0, 3),
        Among(u"\u00F3", 0, 4),
        Among(u"\u00F6", 0, 4),
        Among(u"\u00FA", 0, 5),
        Among(u"\u00FC", 0, 5)
    ]

    a_1 = [
        Among(u"", -1, 3),
        Among(u"I", 0, 2),
        Among(u"Y", 0, 1)
    ]

    a_2 = [
        Among(u"dd", -1, -1),
        Among(u"kk", -1, -1),
        Among(u"tt", -1, -1)
    ]

    a_3 = [
        Among(u"ene", -1, 2),
        Among(u"se", -1, 3),
        Among(u"en", -1, 2),
        Among(u"heden", 2, 1),
        Among(u"s", -1, 3)
    ]

    a_4 = [
        Among(u"end", -1, 1),
        Among(u"ig", -1, 2),
        Among(u"ing", -1, 1),
        Among(u"lijk", -1, 3),
        Among(u"baar", -1, 4),
        Among(u"bar", -1, 5)
    ]

    a_5 = [
        Among(u"aa", -1, -1),
        Among(u"ee", -1, -1),
        Among(u"oo", -1, -1),
        Among(u"uu", -1, -1)
    ]

    g_v = [17, 65, 16, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 128]

    g_v_I = [1, 0, 0, 17, 65, 16, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 128]

    g_v_j = [17, 67, 16, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 128]

    I_p2 = 0
    I_p1 = 0
    B_e_found = False

    def __r_prelude(self):
        v_1 = self.cursor
        while True:
            v_2 = self.cursor
            try:
                self.bra = self.cursor
                among_var = self.find_among(DutchStemmer.a_0)
                if among_var == 0:
                    raise lab0()
                self.ket = self.cursor
                if among_var == 1:
                    if not self.slice_from(u"a"):
                        return False
                elif among_var == 2:
                    if not self.slice_from(u"e"):
                        return False
                elif among_var == 3:
                    if not self.slice_from(u"i"):
                        return False
                elif among_var == 4:
                    if not self.slice_from(u"o"):
                        return False
                elif among_var == 5:
                    if not self.slice_from(u"u"):
                        return False
                else:
                    if self.cursor >= self.limit:
                        raise lab0()
                    self.cursor += 1
                continue
            except lab0: pass
            self.cursor = v_2
            break
        self.cursor = v_1
        v_3 = self.cursor
        try:
            self.bra = self.cursor
            if not self.eq_s(u"y"):
                self.cursor = v_3
                raise lab1()
            self.ket = self.cursor
            if not self.slice_from(u"Y"):
                return False
        except lab1: pass
        while True:
            v_4 = self.cursor
            try:
                try:
                    while True:
                        v_5 = self.cursor
                        try:
                            if not self.in_grouping(DutchStemmer.g_v, 97, 232):
                                raise lab4()
                            self.bra = self.cursor
                            try:
                                v_6 = self.cursor
                                try:
                                    if not self.eq_s(u"i"):
                                        raise lab6()
                                    self.ket = self.cursor
                                    if not self.in_grouping(DutchStemmer.g_v, 97, 232):
                                        raise lab6()
                                    if not self.slice_from(u"I"):
                                        return False
                                    raise lab5()
                                except lab6: pass
                                self.cursor = v_6
                                if not self.eq_s(u"y"):
                                    raise lab4()
                                self.ket = self.cursor
                                if not self.slice_from(u"Y"):
                                    return False
                            except lab5: pass
                            self.cursor = v_5
                            raise lab3()
                        except lab4: pass
                        self.cursor = v_5
                        if self.cursor >= self.limit:
                            raise lab2()
                        self.cursor += 1
                except lab3: pass
                continue
            except lab2: pass
            self.cursor = v_4
            break
        return True

    def __r_mark_regions(self):
        self.I_p1 = self.limit
        self.I_p2 = self.limit
        if not self.go_out_grouping(DutchStemmer.g_v, 97, 232):
            return False
        self.cursor += 1
        if not self.go_in_grouping(DutchStemmer.g_v, 97, 232):
            return False
        self.cursor += 1
        self.I_p1 = self.cursor
        try:
            if not self.I_p1 < 3:
                raise lab0()
            self.I_p1 = 3
        except lab0: pass
        if not self.go_out_grouping(DutchStemmer.g_v, 97, 232):
            return False
        self.cursor += 1
        if not self.go_in_grouping(DutchStemmer.g_v, 97, 232):
            return False
        self.cursor += 1
        self.I_p2 = self.cursor
        return True

    def __r_postlude(self):
        while True:
            v_1 = self.cursor
            try:
                self.bra = self.cursor
                among_var = self.find_among(DutchStemmer.a_1)
                if among_var == 0:
                    raise lab0()
                self.ket = self.cursor
                if among_var == 1:
                    if not self.slice_from(u"y"):
                        return False
                elif among_var == 2:
                    if not self.slice_from(u"i"):
                        return False
                else:
                    if self.cursor >= self.limit:
                        raise lab0()
                    self.cursor += 1
                continue
            except lab0: pass
            self.cursor = v_1
            break
        return True

    def __r_R1(self):
        if not self.I_p1 <= self.cursor:
            return False
        return True

    def __r_R2(self):
        if not self.I_p2 <= self.cursor:
            return False
        return True

    def __r_undouble(self):
        v_1 = self.limit - self.cursor
        if self.find_among_b(DutchStemmer.a_2) == 0:
            return False
        self.cursor = self.limit - v_1
        self.ket = self.cursor
        if self.cursor <= self.limit_backward:
            return False
        self.cursor -= 1
        self.bra = self.cursor
        if not self.slice_del():
            return False

        return True

    def __r_e_ending(self):
        self.B_e_found = False
        self.ket = self.cursor
        if not self.eq_s_b(u"e"):
            return False
        self.bra = self.cursor
        if not self.__r_R1():
            return False
        v_1 = self.limit - self.cursor
        if not self.out_grouping_b(DutchStemmer.g_v, 97, 232):
            return False
        self.cursor = self.limit - v_1
        if not self.slice_del():
            return False

        self.B_e_found = True
        if not self.__r_undouble():
            return False
        return True

    def __r_en_ending(self):
        if not self.__r_R1():
            return False
        v_1 = self.limit - self.cursor
        if not self.out_grouping_b(DutchStemmer.g_v, 97, 232):
            return False
        self.cursor = self.limit - v_1
        v_2 = self.limit - self.cursor
        try:
            if not self.eq_s_b(u"gem"):
                raise lab0()
            return False
        except lab0: pass
        self.cursor = self.limit - v_2
        if not self.slice_del():
            return False

        if not self.__r_undouble():
            return False
        return True

    def __r_standard_suffix(self):
        v_1 = self.limit - self.cursor
        try:
            self.ket = self.cursor
            among_var = self.find_among_b(DutchStemmer.a_3)
            if among_var == 0:
                raise lab0()
            self.bra = self.cursor
            if among_var == 1:
                if not self.__r_R1():
                    raise lab0()
                if not self.slice_from(u"heid"):
                    return False
            elif among_var == 2:
                if not self.__r_en_ending():
                    raise lab0()
            else:
                if not self.__r_R1():
                    raise lab0()
                if not self.out_grouping_b(DutchStemmer.g_v_j, 97, 232):
                    raise lab0()
                if not self.slice_del():
                    return False

        except lab0: pass
        self.cursor = self.limit - v_1
        v_2 = self.limit - self.cursor
        self.__r_e_ending()
        self.cursor = self.limit - v_2
        v_3 = self.limit - self.cursor
        try:
            self.ket = self.cursor
            if not self.eq_s_b(u"heid"):
                raise lab1()
            self.bra = self.cursor
            if not self.__r_R2():
                raise lab1()
            v_4 = self.limit - self.cursor
            try:
                if not self.eq_s_b(u"c"):
                    raise lab2()
                raise lab1()
            except lab2: pass
            self.cursor = self.limit - v_4
            if not self.slice_del():
                return False

            self.ket = self.cursor
            if not self.eq_s_b(u"en"):
                raise lab1()
            self.bra = self.cursor
            if not self.__r_en_ending():
                raise lab1()
        except lab1: pass
        self.cursor = self.limit - v_3
        v_5 = self.limit - self.cursor
        try:
            self.ket = self.cursor
            among_var = self.find_among_b(DutchStemmer.a_4)
            if among_var == 0:
                raise lab3()
            self.bra = self.cursor
            if among_var == 1:
                if not self.__r_R2():
                    raise lab3()
                if not self.slice_del():
                    return False

                try:
                    v_6 = self.limit - self.cursor
                    try:
                        self.ket = self.cursor
                        if not self.eq_s_b(u"ig"):
                            raise lab5()
                        self.bra = self.cursor
                        if not self.__r_R2():
                            raise lab5()
                        v_7 = self.limit - self.cursor
                        try:
                            if not self.eq_s_b(u"e"):
                                raise lab6()
                            raise lab5()
                        except lab6: pass
                        self.cursor = self.limit - v_7
                        if not self.slice_del():
                            return False

                        raise lab4()
                    except lab5: pass
                    self.cursor = self.limit - v_6
                    if not self.__r_undouble():
                        raise lab3()
                except lab4: pass
            elif among_var == 2:
                if not self.__r_R2():
                    raise lab3()
                v_8 = self.limit - self.cursor
                try:
                    if not self.eq_s_b(u"e"):
                        raise lab7()
                    raise lab3()
                except lab7: pass
                self.cursor = self.limit - v_8
                if not self.slice_del():
                    return False

            elif among_var == 3:
                if not self.__r_R2():
                    raise lab3()
                if not self.slice_del():
                    return False

                if not self.__r_e_ending():
                    raise lab3()
            elif among_var == 4:
                if not self.__r_R2():
                    raise lab3()
                if not self.slice_del():
                    return False

            else:
                if not self.__r_R2():
                    raise lab3()
                if not self.B_e_found:
                    raise lab3()
                if not self.slice_del():
                    return False

        except lab3: pass
        self.cursor = self.limit - v_5
        v_9 = self.limit - self.cursor
        try:
            if not self.out_grouping_b(DutchStemmer.g_v_I, 73, 232):
                raise lab8()
            v_10 = self.limit - self.cursor
            if self.find_among_b(DutchStemmer.a_5) == 0:
                raise lab8()
            if not self.out_grouping_b(DutchStemmer.g_v, 97, 232):
                raise lab8()
            self.cursor = self.limit - v_10
            self.ket = self.cursor
            if self.cursor <= self.limit_backward:
                raise lab8()
            self.cursor -= 1
            self.bra = self.cursor
            if not self.slice_del():
                return False

        except lab8: pass
        self.cursor = self.limit - v_9
        return True

    def _stem(self):
        v_1 = self.cursor
        self.__r_prelude()
        self.cursor = v_1
        v_2 = self.cursor
        self.__r_mark_regions()
        self.cursor = v_2
        self.limit_backward = self.cursor
        self.cursor = self.limit
        self.__r_standard_suffix()
        self.cursor = self.limit_backward
        v_4 = self.cursor
        self.__r_postlude()
        self.cursor = v_4
        return True


class lab0(BaseException): pass


class lab1(BaseException): pass


class lab2(BaseException): pass


class lab3(BaseException): pass


class lab4(BaseException): pass


class lab5(BaseException): pass


class lab6(BaseException): pass


class lab7(BaseException): pass


class lab8(BaseException): pass



# Generated by Snowball 2.2.0 - https://snowballstem.org/

from .basestemmer import BaseStemmer
from .among import Among


class PortugueseStemmer(BaseStemmer):
    '''
    This class implements the stemming algorithm defined by a snowball script.
    Generated by Snowball 2.2.0 - https://snowballstem.org/
    '''

    a_0 = [
        Among(u"", -1, 3),
        Among(u"\u00E3", 0, 1),
        Among(u"\u00F5", 0, 2)
    ]

    a_1 = [
        Among(u"", -1, 3),
        Among(u"a~", 0, 1),
        Among(u"o~", 0, 2)
    ]

    a_2 = [
        Among(u"ic", -1, -1),
        Among(u"ad", -1, -1),
        Among(u"os", -1, -1),
        Among(u"iv", -1, 1)
    ]

    a_3 = [
        Among(u"ante", -1, 1),
        Among(u"avel", -1, 1),
        Among(u"\u00EDvel", -1, 1)
    ]

    a_4 = [
        Among(u"ic", -1, 1),
        Among(u"abil", -1, 1),
        Among(u"iv", -1, 1)
    ]

    a_5 = [
        Among(u"ica", -1, 1),
        Among(u"\u00E2ncia", -1, 1),
        Among(u"\u00EAncia", -1, 4),
        Among(u"logia", -1, 2),
        Among(u"ira", -1, 9),
        Among(u"adora", -1, 1),
        Among(u"osa", -1, 1),
        Among(u"ista", -1, 1),
        Among(u"iva", -1, 8),
        Among(u"eza", -1, 1),
        Among(u"idade", -1, 7),
        Among(u"ante", -1, 1),
        Among(u"mente", -1, 6),
        Among(u"amente", 12, 5),
        Among(u"\u00E1vel", -1, 1),
        Among(u"\u00EDvel", -1, 1),
        Among(u"ico", -1, 1),
        Among(u"ismo", -1, 1),
        Among(u"oso", -1, 1),
        Among(u"amento", -1, 1),
        Among(u"imento", -1, 1),
        Among(u"ivo", -1, 8),
        Among(u"a\u00E7a~o", -1, 1),
        Among(u"u\u00E7a~o", -1, 3),
        Among(u"ador", -1, 1),
        Among(u"icas", -1, 1),
        Among(u"\u00EAncias", -1, 4),
        Among(u"logias", -1, 2),
        Among(u"iras", -1, 9),
        Among(u"adoras", -1, 1),
        Among(u"osas", -1, 1),
        Among(u"istas", -1, 1),
        Among(u"ivas", -1, 8),
        Among(u"ezas", -1, 1),
        Among(u"idades", -1, 7),
        Among(u"adores", -1, 1),
        Among(u"antes", -1, 1),
        Among(u"a\u00E7o~es", -1, 1),
        Among(u"u\u00E7o~es", -1, 3),
        Among(u"icos", -1, 1),
        Among(u"ismos", -1, 1),
        Among(u"osos", -1, 1),
        Among(u"amentos", -1, 1),
        Among(u"imentos", -1, 1),
        Among(u"ivos", -1, 8)
    ]

    a_6 = [
        Among(u"ada", -1, 1),
        Among(u"ida", -1, 1),
        Among(u"ia", -1, 1),
        Among(u"aria", 2, 1),
        Among(u"eria", 2, 1),
        Among(u"iria", 2, 1),
        Among(u"ara", -1, 1),
        Among(u"era", -1, 1),
        Among(u"ira", -1, 1),
        Among(u"ava", -1, 1),
        Among(u"asse", -1, 1),
        Among(u"esse", -1, 1),
        Among(u"isse", -1, 1),
        Among(u"aste", -1, 1),
        Among(u"este", -1, 1),
        Among(u"iste", -1, 1),
        Among(u"ei", -1, 1),
        Among(u"arei", 16, 1),
        Among(u"erei", 16, 1),
        Among(u"irei", 16, 1),
        Among(u"am", -1, 1),
        Among(u"iam", 20, 1),
        Among(u"ariam", 21, 1),
        Among(u"eriam", 21, 1),
        Among(u"iriam", 21, 1),
        Among(u"aram", 20, 1),
        Among(u"eram", 20, 1),
        Among(u"iram", 20, 1),
        Among(u"avam", 20, 1),
        Among(u"em", -1, 1),
        Among(u"arem", 29, 1),
        Among(u"erem", 29, 1),
        Among(u"irem", 29, 1),
        Among(u"assem", 29, 1),
        Among(u"essem", 29, 1),
        Among(u"issem", 29, 1),
        Among(u"ado", -1, 1),
        Among(u"ido", -1, 1),
        Among(u"ando", -1, 1),
        Among(u"endo", -1, 1),
        Among(u"indo", -1, 1),
        Among(u"ara~o", -1, 1),
        Among(u"era~o", -1, 1),
        Among(u"ira~o", -1, 1),
        Among(u"ar", -1, 1),
        Among(u"er", -1, 1),
        Among(u"ir", -1, 1),
        Among(u"as", -1, 1),
        Among(u"adas", 47, 1),
        Among(u"idas", 47, 1),
        Among(u"ias", 47, 1),
        Among(u"arias", 50, 1),
        Among(u"erias", 50, 1),
        Among(u"irias", 50, 1),
        Among(u"aras", 47, 1),
        Among(u"eras", 47, 1),
        Among(u"iras", 47, 1),
        Among(u"avas", 47, 1),
        Among(u"es", -1, 1),
        Among(u"ardes", 58, 1),
        Among(u"erdes", 58, 1),
        Among(u"irdes", 58, 1),
        Among(u"ares", 58, 1),
        Among(u"eres", 58, 1),
        Among(u"ires", 58, 1),
        Among(u"asses", 58, 1),
        Among(u"esses", 58, 1),
        Among(u"isses", 58, 1),
        Among(u"astes", 58, 1),
        Among(u"estes", 58, 1),
        Among(u"istes", 58, 1),
        Among(u"is", -1, 1),
        Among(u"ais", 71, 1),
        Among(u"eis", 71, 1),
        Among(u"areis", 73, 1),
        Among(u"ereis", 73, 1),
        Among(u"ireis", 73, 1),
        Among(u"\u00E1reis", 73, 1),
        Among(u"\u00E9reis", 73, 1),
        Among(u"\u00EDreis", 73, 1),
        Among(u"\u00E1sseis", 73, 1),
        Among(u"\u00E9sseis", 73, 1),
        Among(u"\u00EDsseis", 73, 1),
        Among(u"\u00E1veis", 73, 1),
        Among(u"\u00EDeis", 73, 1),
        Among(u"ar\u00EDeis", 84, 1),
        Among(u"er\u00EDeis", 84, 1),
        Among(u"ir\u00EDeis", 84, 1),
        Among(u"ados", -1, 1),
        Among(u"idos", -1, 1),
        Among(u"amos", -1, 1),
        Among(u"\u00E1ramos", 90, 1),
        Among(u"\u00E9ramos", 90, 1),
        Among(u"\u00EDramos", 90, 1),
        Among(u"\u00E1vamos", 90, 1),
        Among(u"\u00EDamos", 90, 1),
        Among(u"ar\u00EDamos", 95, 1),
        Among(u"er\u00EDamos", 95, 1),
        Among(u"ir\u00EDamos", 95, 1),
        Among(u"emos", -1, 1),
        Among(u"aremos", 99, 1),
        Among(u"eremos", 99, 1),
        Among(u"iremos", 99, 1),
        Among(u"\u00E1ssemos", 99, 1),
        Among(u"\u00EAssemos", 99, 1),
        Among(u"\u00EDssemos", 99, 1),
        Among(u"imos", -1, 1),
        Among(u"armos", -1, 1),
        Among(u"ermos", -1, 1),
        Among(u"irmos", -1, 1),
        Among(u"\u00E1mos", -1, 1),
        Among(u"ar\u00E1s", -1, 1),
        Among(u"er\u00E1s", -1, 1),
        Among(u"ir\u00E1s", -1, 1),
        Among(u"eu", -1, 1),
        Among(u"iu", -1, 1),
        Among(u"ou", -1, 1),
        Among(u"ar\u00E1", -1, 1),
        Among(u"er\u00E1", -1, 1),
        Among(u"ir\u00E1", -1, 1)
    ]

    a_7 = [
        Among(u"a", -1, 1),
        Among(u"i", -1, 1),
        Among(u"o", -1, 1),
        Among(u"os", -1, 1),
        Among(u"\u00E1", -1, 1),
        Among(u"\u00ED", -1, 1),
        Among(u"\u00F3", -1, 1)
    ]

    a_8 = [
        Among(u"e", -1, 1),
        Among(u"\u00E7", -1, 2),
        Among(u"\u00E9", -1, 1),
        Among(u"\u00EA", -1, 1)
    ]

    g_v = [17, 65, 16, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3, 19, 12, 2]

    I_p2 = 0
    I_p1 = 0
    I_pV = 0

    def __r_prelude(self):
        while True:
            v_1 = self.cursor
            try:
                self.bra = self.cursor
                among_var = self.find_among(PortugueseStemmer.a_0)
                if among_var == 0:
                    raise lab0()
                self.ket = self.cursor
                if among_var == 1:
                    if not self.slice_from(u"a~"):
                        return False
                elif among_var == 2:
                    if not self.slice_from(u"o~"):
                        return False
                else:
                    if self.cursor >= self.limit:
                        raise lab0()
                    self.cursor += 1
                continue
            except lab0: pass
            self.cursor = v_1
            break
        return True

    def __r_mark_regions(self):
        self.I_pV = self.limit
        self.I_p1 = self.limit
        self.I_p2 = self.limit
        v_1 = self.cursor
        try:
            try:
                v_2 = self.cursor
                try:
                    if not self.in_grouping(PortugueseStemmer.g_v, 97, 250):
                        raise lab2()
                    try:
                        v_3 = self.cursor
                        try:
                            if not self.out_grouping(PortugueseStemmer.g_v, 97, 250):
                                raise lab4()
                            if not self.go_out_grouping(PortugueseStemmer.g_v, 97, 250):
                                raise lab4()
                            self.cursor += 1
                            raise lab3()
                        except lab4: pass
                        self.cursor = v_3
                        if not self.in_grouping(PortugueseStemmer.g_v, 97, 250):
                            raise lab2()
                        if not self.go_in_grouping(PortugueseStemmer.g_v, 97, 250):
                            raise lab2()
                        self.cursor += 1
                    except lab3: pass
                    raise lab1()
                except lab2: pass
                self.cursor = v_2
                if not self.out_grouping(PortugueseStemmer.g_v, 97, 250):
                    raise lab0()
                try:
                    v_4 = self.cursor
                    try:
                        if not self.out_grouping(PortugueseStemmer.g_v, 97, 250):
                            raise lab6()
                        if not self.go_out_grouping(PortugueseStemmer.g_v, 97, 250):
                            raise lab6()
                        self.cursor += 1
                        raise lab5()
                    except lab6: pass
                    self.cursor = v_4
                    if not self.in_grouping(PortugueseStemmer.g_v, 97, 250):
                        raise lab0()
                    if self.cursor >= self.limit:
                        raise lab0()
                    self.cursor += 1
                except lab5: pass
            except lab1: pass
            self.I_pV = self.cursor
        except lab0: pass
        self.cursor = v_1
        v_5 = self.cursor
        try:
            if not self.go_out_grouping(PortugueseStemmer.g_v, 97, 250):
                raise lab7()
            self.cursor += 1
            if not self.go_in_grouping(PortugueseStemmer.g_v, 97, 250):
                raise lab7()
            self.cursor += 1
            self.I_p1 = self.cursor
            if not self.go_out_grouping(PortugueseStemmer.g_v, 97, 250):
                raise lab7()
            self.cursor += 1
            if not self.go_in_grouping(PortugueseStemmer.g_v, 97, 250):
                raise lab7()
            self.cursor += 1
            self.I_p2 = self.cursor
        except lab7: pass
        self.cursor = v_5
        return True

    def __r_postlude(self):
        while True:
            v_1 = self.cursor
            try:
                self.bra = self.cursor
                among_var = self.find_among(PortugueseStemmer.a_1)
                if among_var == 0:
                    raise lab0()
                self.ket = self.cursor
                if among_var == 1:
                    if not self.slice_from(u"\u00E3"):
                        return False
                elif among_var == 2:
                    if not self.slice_from(u"\u00F5"):
                        return False
                else:
                    if self.cursor >= self.limit:
                        raise lab0()
                    self.cursor += 1
                continue
            except lab0: pass
            self.cursor = v_1
            break
        return True

    def __r_RV(self):
        if not self.I_pV <= self.cursor:
            return False
        return True

    def __r_R1(self):
        if not self.I_p1 <= self.cursor:
            return False
        return True

    def __r_R2(self):
        if not self.I_p2 <= self.cursor:
            return False
        return True

    def __r_standard_suffix(self):
        self.ket = self.cursor
        among_var = self.find_among_b(PortugueseStemmer.a_5)
        if among_var == 0:
            return False
        self.bra = self.cursor
        if among_var == 1:
            if not self.__r_R2():
                return False
            if not self.slice_del():
                return False

        elif among_var == 2:
            if not self.__r_R2():
                return False
            if not self.slice_from(u"log"):
                return False
        elif among_var == 3:
            if not self.__r_R2():
                return False
            if not self.slice_from(u"u"):
                return False
        elif among_var == 4:
            if not self.__r_R2():
                return False
            if not self.slice_from(u"ente"):
                return False
        elif among_var == 5:
            if not self.__r_R1():
                return False
            if not self.slice_del():
                return False

            v_1 = self.limit - self.cursor
            try:
                self.ket = self.cursor
                among_var = self.find_among_b(PortugueseStemmer.a_2)
                if among_var == 0:
                    self.cursor = self.limit - v_1
                    raise lab0()
                self.bra = self.cursor
                if not self.__r_R2():
                    self.cursor = self.limit - v_1
                    raise lab0()
                if not self.slice_del():
                    return False

                if among_var == 1:
                    self.ket = self.cursor
                    if not self.eq_s_b(u"at"):
                        self.cursor = self.limit - v_1
                        raise lab0()
                    self.bra = self.cursor
                    if not self.__r_R2():
                        self.cursor = self.limit - v_1
                        raise lab0()
                    if not self.slice_del():
                        return False

            except lab0: pass
        elif among_var == 6:
            if not self.__r_R2():
                return False
            if not self.slice_del():
                return False

            v_2 = self.limit - self.cursor
            try:
                self.ket = self.cursor
                if self.find_among_b(PortugueseStemmer.a_3) == 0:
                    self.cursor = self.limit - v_2
                    raise lab1()
                self.bra = self.cursor
                if not self.__r_R2():
                    self.cursor = self.limit - v_2
                    raise lab1()
                if not self.slice_del():
                    return False

            except lab1: pass
        elif among_var == 7:
            if not self.__r_R2():
                return False
            if not self.slice_del():
                return False

            v_3 = self.limit - self.cursor
            try:
                self.ket = self.cursor
                if self.find_among_b(PortugueseStemmer.a_4) == 0:
                    self.cursor = self.limit - v_3
                    raise lab2()
                self.bra = self.cursor
                if not self.__r_R2():
                    self.cursor = self.limit - v_3
                    raise lab2()
                if not self.slice_del():
                    return False

            except lab2: pass
        elif among_var == 8:
            if not self.__r_R2():
                return False
            if not self.slice_del():
                return False

            v_4 = self.limit - self.cursor
            try:
                self.ket = self.cursor
                if not self.eq_s_b(u"at"):
                    self.cursor = self.limit - v_4
                    raise lab3()
                self.bra = self.cursor
                if not self.__r_R2():
                    self.cursor = self.limit - v_4
                    raise lab3()
                if not self.slice_del():
                    return False

            except lab3: pass
        else:
            if not self.__r_RV():
                return False
            if not self.eq_s_b(u"e"):
                return False
            if not self.slice_from(u"ir"):
                return False
        return True

    def __r_verb_suffix(self):
        if self.cursor < self.I_pV:
            return False
        v_2 = self.limit_backward
        self.limit_backward = self.I_pV
        self.ket = self.cursor
        if self.find_among_b(PortugueseStemmer.a_6) == 0:
            self.limit_backward = v_2
            return False
        self.bra = self.cursor
        if not self.slice_del():
            return False

        self.limit_backward = v_2
        return True

    def __r_residual_suffix(self):
        self.ket = self.cursor
        if self.find_among_b(PortugueseStemmer.a_7) == 0:
            return False
        self.bra = self.cursor
        if not self.__r_RV():
            return False
        if not self.slice_del():
            return False

        return True

    def __r_residual_form(self):
        self.ket = self.cursor
        among_var = self.find_among_b(PortugueseStemmer.a_8)
        if among_var == 0:
            return False
        self.bra = self.cursor
        if among_var == 1:
            if not self.__r_RV():
                return False
            if not self.slice_del():
                return False

            self.ket = self.cursor
            try:
                v_1 = self.limit - self.cursor
                try:
                    if not self.eq_s_b(u"u"):
                        raise lab1()
                    self.bra = self.cursor
                    v_2 = self.limit - self.cursor
                    if not self.eq_s_b(u"g"):
                        raise lab1()
                    self.cursor = self.limit - v_2
                    raise lab0()
                except lab1: pass
                self.cursor = self.limit - v_1
                if not self.eq_s_b(u"i"):
                    return False
                self.bra = self.cursor
                v_3 = self.limit - self.cursor
                if not self.eq_s_b(u"c"):
                    return False
                self.cursor = self.limit - v_3
            except lab0: pass
            if not self.__r_RV():
                return False
            if not self.slice_del():
                return False

        else:
            if not self.slice_from(u"c"):
                return False
        return True

    def _stem(self):
        v_1 = self.cursor
        self.__r_prelude()
        self.cursor = v_1
        self.__r_mark_regions()
        self.limit_backward = self.cursor
        self.cursor = self.limit
        v_3 = self.limit - self.cursor
        try:
            try:
                v_4 = self.limit - self.cursor
                try:
                    v_5 = self.limit - self.cursor
                    try:
                        v_6 = self.limit - self.cursor
                        try:
                            if not self.__r_standard_suffix():
                                raise lab4()
                            raise lab3()
                        except lab4: pass
                        self.cursor = self.limit - v_6
                        if not self.__r_verb_suffix():
                            raise lab2()
                    except lab3: pass
                    self.cursor = self.limit - v_5
                    v_7 = self.limit - self.cursor
                    try:
                        self.ket = self.cursor
                        if not self.eq_s_b(u"i"):
                            raise lab5()
                        self.bra = self.cursor
                        v_8 = self.limit - self.cursor
                        if not self.eq_s_b(u"c"):
                            raise lab5()
                        self.cursor = self.limit - v_8
                        if not self.__r_RV():
                            raise lab5()
                        if not self.slice_del():
                            return False

                    except lab5: pass
                    self.cursor = self.limit - v_7
                    raise lab1()
                except lab2: pass
                self.cursor = self.limit - v_4
                if not self.__r_residual_suffix():
                    raise lab0()
            except lab1: pass
        except lab0: pass
        self.cursor = self.limit - v_3
        v_9 = self.limit - self.cursor
        self.__r_residual_form()
        self.cursor = self.limit - v_9
        self.cursor = self.limit_backward
        v_10 = self.cursor
        self.__r_postlude()
        self.cursor = v_10
        return True


class lab0(BaseException): pass


class lab1(BaseException): pass


class lab2(BaseException): pass


class lab3(BaseException): pass


class lab4(BaseException): pass


class lab5(BaseException): pass


class lab6(BaseException): pass


class lab7(BaseException): pass



# Generated by Snowball 2.2.0 - https://snowballstem.org/

from .basestemmer import BaseStemmer
from .among import Among


class LithuanianStemmer(BaseStemmer):
    '''
    This class implements the stemming algorithm defined by a snowball script.
    Generated by Snowball 2.2.0 - https://snowballstem.org/
    '''

    a_0 = [
        Among(u"a", -1, -1),
        Among(u"ia", 0, -1),
        Among(u"eria", 1, -1),
        Among(u"osna", 0, -1),
        Among(u"iosna", 3, -1),
        Among(u"uosna", 3, -1),
        Among(u"iuosna", 5, -1),
        Among(u"ysna", 0, -1),
        Among(u"\u0117sna", 0, -1),
        Among(u"e", -1, -1),
        Among(u"ie", 9, -1),
        Among(u"enie", 10, -1),
        Among(u"erie", 10, -1),
        Among(u"oje", 9, -1),
        Among(u"ioje", 13, -1),
        Among(u"uje", 9, -1),
        Among(u"iuje", 15, -1),
        Among(u"yje", 9, -1),
        Among(u"enyje", 17, -1),
        Among(u"eryje", 17, -1),
        Among(u"\u0117je", 9, -1),
        Among(u"ame", 9, -1),
        Among(u"iame", 21, -1),
        Among(u"sime", 9, -1),
        Among(u"ome", 9, -1),
        Among(u"\u0117me", 9, -1),
        Among(u"tum\u0117me", 25, -1),
        Among(u"ose", 9, -1),
        Among(u"iose", 27, -1),
        Among(u"uose", 27, -1),
        Among(u"iuose", 29, -1),
        Among(u"yse", 9, -1),
        Among(u"enyse", 31, -1),
        Among(u"eryse", 31, -1),
        Among(u"\u0117se", 9, -1),
        Among(u"ate", 9, -1),
        Among(u"iate", 35, -1),
        Among(u"ite", 9, -1),
        Among(u"kite", 37, -1),
        Among(u"site", 37, -1),
        Among(u"ote", 9, -1),
        Among(u"tute", 9, -1),
        Among(u"\u0117te", 9, -1),
        Among(u"tum\u0117te", 42, -1),
        Among(u"i", -1, -1),
        Among(u"ai", 44, -1),
        Among(u"iai", 45, -1),
        Among(u"eriai", 46, -1),
        Among(u"ei", 44, -1),
        Among(u"tumei", 48, -1),
        Among(u"ki", 44, -1),
        Among(u"imi", 44, -1),
        Among(u"erimi", 51, -1),
        Among(u"umi", 44, -1),
        Among(u"iumi", 53, -1),
        Among(u"si", 44, -1),
        Among(u"asi", 55, -1),
        Among(u"iasi", 56, -1),
        Among(u"esi", 55, -1),
        Among(u"iesi", 58, -1),
        Among(u"siesi", 59, -1),
        Among(u"isi", 55, -1),
        Among(u"aisi", 61, -1),
        Among(u"eisi", 61, -1),
        Among(u"tumeisi", 63, -1),
        Among(u"uisi", 61, -1),
        Among(u"osi", 55, -1),
        Among(u"\u0117josi", 66, -1),
        Among(u"uosi", 66, -1),
        Among(u"iuosi", 68, -1),
        Among(u"siuosi", 69, -1),
        Among(u"usi", 55, -1),
        Among(u"ausi", 71, -1),
        Among(u"\u010Diausi", 72, -1),
        Among(u"\u0105si", 55, -1),
        Among(u"\u0117si", 55, -1),
        Among(u"\u0173si", 55, -1),
        Among(u"t\u0173si", 76, -1),
        Among(u"ti", 44, -1),
        Among(u"enti", 78, -1),
        Among(u"inti", 78, -1),
        Among(u"oti", 78, -1),
        Among(u"ioti", 81, -1),
        Among(u"uoti", 81, -1),
        Among(u"iuoti", 83, -1),
        Among(u"auti", 78, -1),
        Among(u"iauti", 85, -1),
        Among(u"yti", 78, -1),
        Among(u"\u0117ti", 78, -1),
        Among(u"tel\u0117ti", 88, -1),
        Among(u"in\u0117ti", 88, -1),
        Among(u"ter\u0117ti", 88, -1),
        Among(u"ui", 44, -1),
        Among(u"iui", 92, -1),
        Among(u"eniui", 93, -1),
        Among(u"oj", -1, -1),
        Among(u"\u0117j", -1, -1),
        Among(u"k", -1, -1),
        Among(u"am", -1, -1),
        Among(u"iam", 98, -1),
        Among(u"iem", -1, -1),
        Among(u"im", -1, -1),
        Among(u"sim", 101, -1),
        Among(u"om", -1, -1),
        Among(u"tum", -1, -1),
        Among(u"\u0117m", -1, -1),
        Among(u"tum\u0117m", 105, -1),
        Among(u"an", -1, -1),
        Among(u"on", -1, -1),
        Among(u"ion", 108, -1),
        Among(u"un", -1, -1),
        Among(u"iun", 110, -1),
        Among(u"\u0117n", -1, -1),
        Among(u"o", -1, -1),
        Among(u"io", 113, -1),
        Among(u"enio", 114, -1),
        Among(u"\u0117jo", 113, -1),
        Among(u"uo", 113, -1),
        Among(u"s", -1, -1),
        Among(u"as", 118, -1),
        Among(u"ias", 119, -1),
        Among(u"es", 118, -1),
        Among(u"ies", 121, -1),
        Among(u"is", 118, -1),
        Among(u"ais", 123, -1),
        Among(u"iais", 124, -1),
        Among(u"tumeis", 123, -1),
        Among(u"imis", 123, -1),
        Among(u"enimis", 127, -1),
        Among(u"omis", 123, -1),
        Among(u"iomis", 129, -1),
        Among(u"umis", 123, -1),
        Among(u"\u0117mis", 123, -1),
        Among(u"enis", 123, -1),
        Among(u"asis", 123, -1),
        Among(u"ysis", 123, -1),
        Among(u"ams", 118, -1),
        Among(u"iams", 136, -1),
        Among(u"iems", 118, -1),
        Among(u"ims", 118, -1),
        Among(u"enims", 139, -1),
        Among(u"erims", 139, -1),
        Among(u"oms", 118, -1),
        Among(u"ioms", 142, -1),
        Among(u"ums", 118, -1),
        Among(u"\u0117ms", 118, -1),
        Among(u"ens", 118, -1),
        Among(u"os", 118, -1),
        Among(u"ios", 147, -1),
        Among(u"uos", 147, -1),
        Among(u"iuos", 149, -1),
        Among(u"ers", 118, -1),
        Among(u"us", 118, -1),
        Among(u"aus", 152, -1),
        Among(u"iaus", 153, -1),
        Among(u"ius", 152, -1),
        Among(u"ys", 118, -1),
        Among(u"enys", 156, -1),
        Among(u"erys", 156, -1),
        Among(u"\u0105s", 118, -1),
        Among(u"i\u0105s", 159, -1),
        Among(u"\u0117s", 118, -1),
        Among(u"am\u0117s", 161, -1),
        Among(u"iam\u0117s", 162, -1),
        Among(u"im\u0117s", 161, -1),
        Among(u"kim\u0117s", 164, -1),
        Among(u"sim\u0117s", 164, -1),
        Among(u"om\u0117s", 161, -1),
        Among(u"\u0117m\u0117s", 161, -1),
        Among(u"tum\u0117m\u0117s", 168, -1),
        Among(u"at\u0117s", 161, -1),
        Among(u"iat\u0117s", 170, -1),
        Among(u"sit\u0117s", 161, -1),
        Among(u"ot\u0117s", 161, -1),
        Among(u"\u0117t\u0117s", 161, -1),
        Among(u"tum\u0117t\u0117s", 174, -1),
        Among(u"\u012Fs", 118, -1),
        Among(u"\u016Bs", 118, -1),
        Among(u"t\u0173s", 118, -1),
        Among(u"at", -1, -1),
        Among(u"iat", 179, -1),
        Among(u"it", -1, -1),
        Among(u"sit", 181, -1),
        Among(u"ot", -1, -1),
        Among(u"\u0117t", -1, -1),
        Among(u"tum\u0117t", 184, -1),
        Among(u"u", -1, -1),
        Among(u"au", 186, -1),
        Among(u"iau", 187, -1),
        Among(u"\u010Diau", 188, -1),
        Among(u"iu", 186, -1),
        Among(u"eniu", 190, -1),
        Among(u"siu", 190, -1),
        Among(u"y", -1, -1),
        Among(u"\u0105", -1, -1),
        Among(u"i\u0105", 194, -1),
        Among(u"\u0117", -1, -1),
        Among(u"\u0119", -1, -1),
        Among(u"\u012F", -1, -1),
        Among(u"en\u012F", 198, -1),
        Among(u"er\u012F", 198, -1),
        Among(u"\u0173", -1, -1),
        Among(u"i\u0173", 201, -1),
        Among(u"er\u0173", 201, -1)
    ]

    a_1 = [
        Among(u"ing", -1, -1),
        Among(u"aj", -1, -1),
        Among(u"iaj", 1, -1),
        Among(u"iej", -1, -1),
        Among(u"oj", -1, -1),
        Among(u"ioj", 4, -1),
        Among(u"uoj", 4, -1),
        Among(u"iuoj", 6, -1),
        Among(u"auj", -1, -1),
        Among(u"\u0105j", -1, -1),
        Among(u"i\u0105j", 9, -1),
        Among(u"\u0117j", -1, -1),
        Among(u"\u0173j", -1, -1),
        Among(u"i\u0173j", 12, -1),
        Among(u"ok", -1, -1),
        Among(u"iok", 14, -1),
        Among(u"iuk", -1, -1),
        Among(u"uliuk", 16, -1),
        Among(u"u\u010Diuk", 16, -1),
        Among(u"i\u0161k", -1, -1),
        Among(u"iul", -1, -1),
        Among(u"yl", -1, -1),
        Among(u"\u0117l", -1, -1),
        Among(u"am", -1, -1),
        Among(u"dam", 23, -1),
        Among(u"jam", 23, -1),
        Among(u"zgan", -1, -1),
        Among(u"ain", -1, -1),
        Among(u"esn", -1, -1),
        Among(u"op", -1, -1),
        Among(u"iop", 29, -1),
        Among(u"ias", -1, -1),
        Among(u"ies", -1, -1),
        Among(u"ais", -1, -1),
        Among(u"iais", 33, -1),
        Among(u"os", -1, -1),
        Among(u"ios", 35, -1),
        Among(u"uos", 35, -1),
        Among(u"iuos", 37, -1),
        Among(u"aus", -1, -1),
        Among(u"iaus", 39, -1),
        Among(u"\u0105s", -1, -1),
        Among(u"i\u0105s", 41, -1),
        Among(u"\u0119s", -1, -1),
        Among(u"ut\u0117ait", -1, -1),
        Among(u"ant", -1, -1),
        Among(u"iant", 45, -1),
        Among(u"siant", 46, -1),
        Among(u"int", -1, -1),
        Among(u"ot", -1, -1),
        Among(u"uot", 49, -1),
        Among(u"iuot", 50, -1),
        Among(u"yt", -1, -1),
        Among(u"\u0117t", -1, -1),
        Among(u"yk\u0161t", -1, -1),
        Among(u"iau", -1, -1),
        Among(u"dav", -1, -1),
        Among(u"sv", -1, -1),
        Among(u"\u0161v", -1, -1),
        Among(u"yk\u0161\u010D", -1, -1),
        Among(u"\u0119", -1, -1),
        Among(u"\u0117j\u0119", 60, -1)
    ]

    a_2 = [
        Among(u"ojime", -1, 7),
        Among(u"\u0117jime", -1, 3),
        Among(u"avime", -1, 6),
        Among(u"okate", -1, 8),
        Among(u"aite", -1, 1),
        Among(u"uote", -1, 2),
        Among(u"asius", -1, 5),
        Among(u"okat\u0117s", -1, 8),
        Among(u"ait\u0117s", -1, 1),
        Among(u"uot\u0117s", -1, 2),
        Among(u"esiu", -1, 4)
    ]

    a_3 = [
        Among(u"\u010D", -1, 1),
        Among(u"d\u017E", -1, 2)
    ]

    a_4 = [
        Among(u"gd", -1, 1)
    ]

    g_v = [17, 65, 16, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 16, 0, 64, 1, 0, 64, 0, 0, 0, 0, 0, 0, 0, 4, 4]

    I_p1 = 0

    def __r_R1(self):
        if not self.I_p1 <= self.cursor:
            return False
        return True

    def __r_step1(self):
        if self.cursor < self.I_p1:
            return False
        v_2 = self.limit_backward
        self.limit_backward = self.I_p1
        self.ket = self.cursor
        if self.find_among_b(LithuanianStemmer.a_0) == 0:
            self.limit_backward = v_2
            return False
        self.bra = self.cursor
        self.limit_backward = v_2
        if not self.__r_R1():
            return False
        if not self.slice_del():
            return False

        return True

    def __r_step2(self):
        while True:
            v_1 = self.limit - self.cursor
            try:
                if self.cursor < self.I_p1:
                    raise lab0()
                v_3 = self.limit_backward
                self.limit_backward = self.I_p1
                self.ket = self.cursor
                if self.find_among_b(LithuanianStemmer.a_1) == 0:
                    self.limit_backward = v_3
                    raise lab0()
                self.bra = self.cursor
                self.limit_backward = v_3
                if not self.slice_del():
                    return False

                continue
            except lab0: pass
            self.cursor = self.limit - v_1
            break
        return True

    def __r_fix_conflicts(self):
        self.ket = self.cursor
        among_var = self.find_among_b(LithuanianStemmer.a_2)
        if among_var == 0:
            return False
        self.bra = self.cursor
        if among_var == 1:
            if not self.slice_from(u"ait\u0117"):
                return False
        elif among_var == 2:
            if not self.slice_from(u"uot\u0117"):
                return False
        elif among_var == 3:
            if not self.slice_from(u"\u0117jimas"):
                return False
        elif among_var == 4:
            if not self.slice_from(u"esys"):
                return False
        elif among_var == 5:
            if not self.slice_from(u"asys"):
                return False
        elif among_var == 6:
            if not self.slice_from(u"avimas"):
                return False
        elif among_var == 7:
            if not self.slice_from(u"ojimas"):
                return False
        else:
            if not self.slice_from(u"okat\u0117"):
                return False
        return True

    def __r_fix_chdz(self):
        self.ket = self.cursor
        among_var = self.find_among_b(LithuanianStemmer.a_3)
        if among_var == 0:
            return False
        self.bra = self.cursor
        if among_var == 1:
            if not self.slice_from(u"t"):
                return False
        else:
            if not self.slice_from(u"d"):
                return False
        return True

    def __r_fix_gd(self):
        self.ket = self.cursor
        if self.find_among_b(LithuanianStemmer.a_4) == 0:
            return False
        self.bra = self.cursor
        if not self.slice_from(u"g"):
            return False
        return True

    def _stem(self):
        self.I_p1 = self.limit
        v_1 = self.cursor
        try:
            v_2 = self.cursor
            try:
                v_3 = self.cursor
                if not self.eq_s(u"a"):
                    self.cursor = v_2
                    raise lab1()
                self.cursor = v_3
                if not len(self.current) > 6:
                    self.cursor = v_2
                    raise lab1()
                c = self.cursor + 1
                if c > self.limit:
                    self.cursor = v_2
                    raise lab1()
                self.cursor = c
            except lab1: pass
            if not self.go_out_grouping(LithuanianStemmer.g_v, 97, 371):
                raise lab0()
            self.cursor += 1
            if not self.go_in_grouping(LithuanianStemmer.g_v, 97, 371):
                raise lab0()
            self.cursor += 1
            self.I_p1 = self.cursor
        except lab0: pass
        self.cursor = v_1
        self.limit_backward = self.cursor
        self.cursor = self.limit
        v_4 = self.limit - self.cursor
        self.__r_fix_conflicts()
        self.cursor = self.limit - v_4
        v_5 = self.limit - self.cursor
        self.__r_step1()
        self.cursor = self.limit - v_5
        v_6 = self.limit - self.cursor
        self.__r_fix_chdz()
        self.cursor = self.limit - v_6
        v_7 = self.limit - self.cursor
        self.__r_step2()
        self.cursor = self.limit - v_7
        v_8 = self.limit - self.cursor
        self.__r_fix_chdz()
        self.cursor = self.limit - v_8
        v_9 = self.limit - self.cursor
        self.__r_fix_gd()
        self.cursor = self.limit - v_9
        self.cursor = self.limit_backward
        return True


class lab0(BaseException): pass


class lab1(BaseException): pass



# Generated by Snowball 2.2.0 - https://snowballstem.org/

from .basestemmer import BaseStemmer
from .among import Among


class GermanStemmer(BaseStemmer):
    '''
    This class implements the stemming algorithm defined by a snowball script.
    Generated by Snowball 2.2.0 - https://snowballstem.org/
    '''

    a_0 = [
        Among(u"", -1, 5),
        Among(u"U", 0, 2),
        Among(u"Y", 0, 1),
        Among(u"\u00E4", 0, 3),
        Among(u"\u00F6", 0, 4),
        Among(u"\u00FC", 0, 2)
    ]

    a_1 = [
        Among(u"e", -1, 2),
        Among(u"em", -1, 1),
        Among(u"en", -1, 2),
        Among(u"ern", -1, 1),
        Among(u"er", -1, 1),
        Among(u"s", -1, 3),
        Among(u"es", 5, 2)
    ]

    a_2 = [
        Among(u"en", -1, 1),
        Among(u"er", -1, 1),
        Among(u"st", -1, 2),
        Among(u"est", 2, 1)
    ]

    a_3 = [
        Among(u"ig", -1, 1),
        Among(u"lich", -1, 1)
    ]

    a_4 = [
        Among(u"end", -1, 1),
        Among(u"ig", -1, 2),
        Among(u"ung", -1, 1),
        Among(u"lich", -1, 3),
        Among(u"isch", -1, 2),
        Among(u"ik", -1, 2),
        Among(u"heit", -1, 3),
        Among(u"keit", -1, 4)
    ]

    g_v = [17, 65, 16, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 8, 0, 32, 8]

    g_s_ending = [117, 30, 5]

    g_st_ending = [117, 30, 4]

    I_x = 0
    I_p2 = 0
    I_p1 = 0

    def __r_prelude(self):
        v_1 = self.cursor
        while True:
            v_2 = self.cursor
            try:
                try:
                    v_3 = self.cursor
                    try:
                        self.bra = self.cursor
                        if not self.eq_s(u"\u00DF"):
                            raise lab2()
                        self.ket = self.cursor
                        if not self.slice_from(u"ss"):
                            return False
                        raise lab1()
                    except lab2: pass
                    self.cursor = v_3
                    if self.cursor >= self.limit:
                        raise lab0()
                    self.cursor += 1
                except lab1: pass
                continue
            except lab0: pass
            self.cursor = v_2
            break
        self.cursor = v_1
        while True:
            v_4 = self.cursor
            try:
                try:
                    while True:
                        v_5 = self.cursor
                        try:
                            if not self.in_grouping(GermanStemmer.g_v, 97, 252):
                                raise lab5()
                            self.bra = self.cursor
                            try:
                                v_6 = self.cursor
                                try:
                                    if not self.eq_s(u"u"):
                                        raise lab7()
                                    self.ket = self.cursor
                                    if not self.in_grouping(GermanStemmer.g_v, 97, 252):
                                        raise lab7()
                                    if not self.slice_from(u"U"):
                                        return False
                                    raise lab6()
                                except lab7: pass
                                self.cursor = v_6
                                if not self.eq_s(u"y"):
                                    raise lab5()
                                self.ket = self.cursor
                                if not self.in_grouping(GermanStemmer.g_v, 97, 252):
                                    raise lab5()
                                if not self.slice_from(u"Y"):
                                    return False
                            except lab6: pass
                            self.cursor = v_5
                            raise lab4()
                        except lab5: pass
                        self.cursor = v_5
                        if self.cursor >= self.limit:
                            raise lab3()
                        self.cursor += 1
                except lab4: pass
                continue
            except lab3: pass
            self.cursor = v_4
            break
        return True

    def __r_mark_regions(self):
        self.I_p1 = self.limit
        self.I_p2 = self.limit
        v_1 = self.cursor
        c = self.cursor + 3
        if c > self.limit:
            return False
        self.cursor = c
        self.I_x = self.cursor
        self.cursor = v_1
        if not self.go_out_grouping(GermanStemmer.g_v, 97, 252):
            return False
        self.cursor += 1
        if not self.go_in_grouping(GermanStemmer.g_v, 97, 252):
            return False
        self.cursor += 1
        self.I_p1 = self.cursor
        try:
            if not self.I_p1 < self.I_x:
                raise lab0()
            self.I_p1 = self.I_x
        except lab0: pass
        if not self.go_out_grouping(GermanStemmer.g_v, 97, 252):
            return False
        self.cursor += 1
        if not self.go_in_grouping(GermanStemmer.g_v, 97, 252):
            return False
        self.cursor += 1
        self.I_p2 = self.cursor
        return True

    def __r_postlude(self):
        while True:
            v_1 = self.cursor
            try:
                self.bra = self.cursor
                among_var = self.find_among(GermanStemmer.a_0)
                if among_var == 0:
                    raise lab0()
                self.ket = self.cursor
                if among_var == 1:
                    if not self.slice_from(u"y"):
                        return False
                elif among_var == 2:
                    if not self.slice_from(u"u"):
                        return False
                elif among_var == 3:
                    if not self.slice_from(u"a"):
                        return False
                elif among_var == 4:
                    if not self.slice_from(u"o"):
                        return False
                else:
                    if self.cursor >= self.limit:
                        raise lab0()
                    self.cursor += 1
                continue
            except lab0: pass
            self.cursor = v_1
            break
        return True

    def __r_R1(self):
        if not self.I_p1 <= self.cursor:
            return False
        return True

    def __r_R2(self):
        if not self.I_p2 <= self.cursor:
            return False
        return True

    def __r_standard_suffix(self):
        v_1 = self.limit - self.cursor
        try:
            self.ket = self.cursor
            among_var = self.find_among_b(GermanStemmer.a_1)
            if among_var == 0:
                raise lab0()
            self.bra = self.cursor
            if not self.__r_R1():
                raise lab0()
            if among_var == 1:
                if not self.slice_del():
                    return False

            elif among_var == 2:
                if not self.slice_del():
                    return False

                v_2 = self.limit - self.cursor
                try:
                    self.ket = self.cursor
                    if not self.eq_s_b(u"s"):
                        self.cursor = self.limit - v_2
                        raise lab1()
                    self.bra = self.cursor
                    if not self.eq_s_b(u"nis"):
                        self.cursor = self.limit - v_2
                        raise lab1()
                    if not self.slice_del():
                        return False

                except lab1: pass
            else:
                if not self.in_grouping_b(GermanStemmer.g_s_ending, 98, 116):
                    raise lab0()
                if not self.slice_del():
                    return False

        except lab0: pass
        self.cursor = self.limit - v_1
        v_3 = self.limit - self.cursor
        try:
            self.ket = self.cursor
            among_var = self.find_among_b(GermanStemmer.a_2)
            if among_var == 0:
                raise lab2()
            self.bra = self.cursor
            if not self.__r_R1():
                raise lab2()
            if among_var == 1:
                if not self.slice_del():
                    return False

            else:
                if not self.in_grouping_b(GermanStemmer.g_st_ending, 98, 116):
                    raise lab2()
                c = self.cursor - 3
                if c < self.limit_backward:
                    raise lab2()
                self.cursor = c
                if not self.slice_del():
                    return False

        except lab2: pass
        self.cursor = self.limit - v_3
        v_4 = self.limit - self.cursor
        try:
            self.ket = self.cursor
            among_var = self.find_among_b(GermanStemmer.a_4)
            if among_var == 0:
                raise lab3()
            self.bra = self.cursor
            if not self.__r_R2():
                raise lab3()
            if among_var == 1:
                if not self.slice_del():
                    return False

                v_5 = self.limit - self.cursor
                try:
                    self.ket = self.cursor
                    if not self.eq_s_b(u"ig"):
                        self.cursor = self.limit - v_5
                        raise lab4()
                    self.bra = self.cursor
                    v_6 = self.limit - self.cursor
                    try:
                        if not self.eq_s_b(u"e"):
                            raise lab5()
                        self.cursor = self.limit - v_5
                        raise lab4()
                    except lab5: pass
                    self.cursor = self.limit - v_6
                    if not self.__r_R2():
                        self.cursor = self.limit - v_5
                        raise lab4()
                    if not self.slice_del():
                        return False

                except lab4: pass
            elif among_var == 2:
                v_7 = self.limit - self.cursor
                try:
                    if not self.eq_s_b(u"e"):
                        raise lab6()
                    raise lab3()
                except lab6: pass
                self.cursor = self.limit - v_7
                if not self.slice_del():
                    return False

            elif among_var == 3:
                if not self.slice_del():
                    return False

                v_8 = self.limit - self.cursor
                try:
                    self.ket = self.cursor
                    try:
                        v_9 = self.limit - self.cursor
                        try:
                            if not self.eq_s_b(u"er"):
                                raise lab9()
                            raise lab8()
                        except lab9: pass
                        self.cursor = self.limit - v_9
                        if not self.eq_s_b(u"en"):
                            self.cursor = self.limit - v_8
                            raise lab7()
                    except lab8: pass
                    self.bra = self.cursor
                    if not self.__r_R1():
                        self.cursor = self.limit - v_8
                        raise lab7()
                    if not self.slice_del():
                        return False

                except lab7: pass
            else:
                if not self.slice_del():
                    return False

                v_10 = self.limit - self.cursor
                try:
                    self.ket = self.cursor
                    if self.find_among_b(GermanStemmer.a_3) == 0:
                        self.cursor = self.limit - v_10
                        raise lab10()
                    self.bra = self.cursor
                    if not self.__r_R2():
                        self.cursor = self.limit - v_10
                        raise lab10()
                    if not self.slice_del():
                        return False

                except lab10: pass
        except lab3: pass
        self.cursor = self.limit - v_4
        return True

    def _stem(self):
        v_1 = self.cursor
        self.__r_prelude()
        self.cursor = v_1
        v_2 = self.cursor
        self.__r_mark_regions()
        self.cursor = v_2
        self.limit_backward = self.cursor
        self.cursor = self.limit
        self.__r_standard_suffix()
        self.cursor = self.limit_backward
        v_4 = self.cursor
        self.__r_postlude()
        self.cursor = v_4
        return True


class lab0(BaseException): pass


class lab1(BaseException): pass


class lab2(BaseException): pass


class lab3(BaseException): pass


class lab4(BaseException): pass


class lab5(BaseException): pass


class lab6(BaseException): pass


class lab7(BaseException): pass


class lab8(BaseException): pass


class lab9(BaseException): pass


class lab10(BaseException): pass
